#!/usr/bin/env node

var fs = require('fs');
var heapdump = require('heapdump');
var nodereport = require('node-report');
var jsonfile = require('jsonfile');
var url = require('url');
var ejs = require('ejs');

var ybyLib = require('yby');
var ybyInst = ybyLib.createInstance({appid: '1', logLevel: ybyLib.LOG_ERROR});

var configFilePath = '/tmp/middleware_config.json';

module.exports = function(config) {
    configFilePath = config.configFile || configFilePath;
    config.outputDir = config.outputDir || '/tmp/';
    config.endPoint = config.endPoint || '/getlist';
    config.enableFw = config.enableFw || false;
    config.enableEndPoint = config.enableEndPoint || false;

    if(config.enableFw === true){
        fs.watchFile(configFilePath, function(event, filename) {
            if(filename){
                console.log('Config file changed !!!');
                try{
                    jsonfile.readFile(configFilePath, function(err, obj) {

                        function generateHeapDump(){
                            {
                                heapdump.writeSnapshot(obj.defaultOutputDir+'HeapDump' + Date.now() + '.heapsnapshot');
                                console.log('Took heapdump');
                            }
                        }

                        if (err){
                            console.log('err in reading file');
                            return;
                        }
                        if(obj.type === 'heap'){
                            for (var i = 0; i < obj.nTimes ; i++){
                                setTimeout(generateHeapDump, (obj.nIntervals*(i+1)));
                            }
                        }
                        else if (obj.type === 'report'){
                            nodereport.setFileName("Nodejs-report" +Date.now());
                            nodereport.setDirectory(obj.defaultOutputDir);
                            nodereport.triggerReport();
                            console.log('Took report');
                        }
                        else{
                            console.log('Enter proper command from CLI');
                        }
                    });
                }
                catch(e){
                    console.log('Caught Exception ' +(e.stack || e.toString()));
                }
            }
            else{
                console.log('Filename not provided');
            }
        });
    }



    return function(req, res, next) {
        //enter bouncer
        var remoteAddress = req.headers['y-ra'] || req.headers.yahooremoteip || req.connection.remoteAddress;
        var YBY = req.headers.cookies.yby || req.headers.cookies.YBY;
        var SYBY = req.headers.cookies.syby || req.headers.cookies.SYBY;
        var ybyCookie = ybyInst.parseAndValidate(YBY, remoteAddress, SYBY);
        if (ybyCookie.getStatus()) {
            console.log('userid:' + ybyCookie.getUserId());
            console.log('expires:' + ybyCookie.getExpirationDate());
            console.log('roles:' + ybyCookie.getRoles());
        }



        req.parsedUrl = req.parsedUrl || url.parse(req.url, true);

        if(req.query.file && config.enableEndPoint === true){
            console.log('Downloading file to ' + config.outputDir);
            res.download(config.outputDir+req.query.file);
        }
        else if(req.parsedUrl.pathname === config.endPoint && config.enableEndPoint === true){
            console.log('Reading list of heapdump and node reports from '+config.outputDir);
            fs.readdir(config.outputDir, function (err, filenames) {
                if (err) {
                    console.log(err);
                    res.writeHead(500, {
                        'Content-Type' : 'text/plain'
                    });
                    res.end('Internal Server Error');
                }
                var heapdump = [];
                var report = [];
                filenames.forEach(function (filename) {
                    if (filename.indexOf("heapsnapshot")>-1) {
                        heapdump.push(filename);

                    }
                    else if (filename.indexOf("Nodejs-report")>-1) {
                        report.push(filename);
                    }
                });
                res.writeHead(200, {
                    'Content-Type' : 'text/html'
                });
                var ejsStr = fs.readFileSync(__dirname+'/main.ejs','utf8');
                res.end(ejs.render(ejsStr,{title:"HEAPDUMP and NODE REPORT FOR NODEJS APPLICATIONS",heapdumpList:heapdump,reportList:report,endPoint:config.endPoint }));
            });

        }

        else{
            next();
        }
    };
};

